﻿------------------------------------------------------------------------------------------------------
-- Initialise the variable we will be working with
------------------------------------------------------------------------------------------------------
local L = LibStub("AceLocale-3.0"):GetLocale("InstanceInfo");
--local Tablet = AceLibrary("Tablet-2.0");
local AceConfig = LibStub("AceConfigDialog-3.0");
local QTC = LibStub:GetLibrary("LibQTipClick-1.1");

InstanceInfo = LibStub("AceAddon-3.0"):NewAddon("InstanceInfo", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0");

local tooltip;

------------------------------------------------------------------------------------------------------
-- Find the hex colour for each class
------------------------------------------------------------------------------------------------------
function InstanceInfo:GetHexColor(class)
	return string.format("%02x%02x%02x", 
		RAID_CLASS_COLORS[class].r * 255,
		RAID_CLASS_COLORS[class].g * 255,
		RAID_CLASS_COLORS[class].b * 255
	);
end


------------------------------------------------------------------------------------------------------
-- Colouring functions
------------------------------------------------------------------------------------------------------
function InstanceInfo_Red( text )
	return "|cffff0000" .. text .. "|r"
end
function InstanceInfo_Yellow( text )
	return "|cffffff00" .. text .. "|r"
end
function InstanceInfo_Green( text )
	return "|cff00ff00" .. text .. "|r"
end
function InstanceInfo_White( text )
	return "|cffffffff" .. text .. "|r"
end

------------------------------------------------------------------------------------------------------
-- Hides the tooltip
------------------------------------------------------------------------------------------------------
local function HideTooltip()
	tooltip:SetScript( "OnLeave", nil )
	tooltip:Hide()
end

------------------------------------------------------------------------------------------------------
-- Function to get and display saved instances for the specified player
------------------------------------------------------------------------------------------------------
local function ShowSavedInstanceInfo(player)
	if InstanceInfo.db.realm.info[player] == nil then
		-- We have no info for this character
		return;
	end
	local info = InstanceInfo.db.realm.info[player];
	
	tooltip:AddNormalLine(
		"[|cff"..InstanceInfo.classColours[info.engClass]..info.player.."|r]",
		L["Zone"],
		L["Raid ID"],
		L["Reset Time"]
	);

	local nMax = #(info.instances);
	for i = 1, nMax do
		local instance = info.instances[i];
		if (instance ~= nil) then
			if (time() <= (info.curtime + instance.remaining)) then
				
				local name = instance.name
				if instance.difficulty > 1 then
					name = instance.name.." ("..DUNGEON_DIFFICULTY2..")"
				end
				
				tooltip:AddNormalLine(
					"",
					name,
					instance.ID,
					SecondsToTime((info.curtime + instance.remaining) - time(), true, false, 4)
				);
			else
				-- This timer expired
				InstanceInfo.db.realm.info[player].instances[i] = nil;
				self:UpdateBrokerText();
			end
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Registers the tooltip
------------------------------------------------------------------------------------------------------
local function drawTooltip()
	-- Begin defining some local variables
	local numInstances = 0;
	local t = {}
	local numCols;

	-- Make sure we don't have any other tooltips
	tooltip:Hide();
	tooltip:Clear();

	if (InstanceInfo.db.profile.showSelf) then
		 numInstances = GetNumSavedInstances();
	else
		for player, playerinfo in pairs(InstanceInfo.db.realm.info) do
			if (playerinfo ~= nil) then
				local nMax = #(playerinfo.instances);
				for i = 1, nMax do
					local instance = playerinfo.instances[i];
					if (instance ~= nil) then
						numInstances = numInstances + 1;
					end
				end
			end
		end
	end
	
	-- Align player name to the left
	t[1] = "LEFT"

	if (numInstances == 0) then
		-- Only show the one column
		numCols = 1;
	else
		-- Align all other columns centered
		t[2] = "CENTER";
		t[3] = "CENTER";
		t[4] = "CENTER";
		
		-- Display all 4 columns
		numCols = 4;
	end
	
	-- Finally create the layout
	tooltip:SetColumnLayout(numCols, unpack(t))
	
	-- What line are we working with?
	local lineNum = tooltip:AddNormalHeader("b")
	
	-- Set the cell info for this line
	tooltip:SetCell(lineNum, 1, InstanceInfo_White(L["InstanceInfo"]), "CENTER", numCols)

	-- Add a blank, dummy line
	lineNum = tooltip:AddLine(" ")

	if (numInstances == 0) then
		-- We're not saved to anything
		tooltip:AddNormalLine(InstanceInfo_White(L["No Saved Instances"]));
	else
		-- Display all our saved instances
		if (not InstanceInfo.db.profile.showSelf) then
			for player, playerinfo in pairs(InstanceInfo.db.realm.info) do
				if (playerinfo ~= nil and playerinfo ~= {}) then
					ShowSavedInstanceInfo(player);
				end
			end
		else
			-- We're only showing ourselves
			ShowSavedInstanceInfo(UnitName("player"));
		end
	end

	-- Add menu items
	tooltip:AddNormalLine(" ")
	lineNum = tooltip:AddNormalLine("c")
	tooltip:SetCell(lineNum, 1, InstanceInfo_Yellow(L["Right-click|r to open the options menu"]), "LEFT", numCols)
end

------------------------------------------------------------------------------------------------------
-- Inits the data object
------------------------------------------------------------------------------------------------------
local dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("InstanceInfo", {
	type = "data source",
	label = "InstanceInfo",
	OnClick = function(_, msg)
		if msg == "LeftButton" then
			-- Open the Raid frame
			ToggleFriendsFrame(5);
		elseif msg == "RightButton" then
			InterfaceOptionsFrame_OpenToCategory("InstanceInfo")
		end
	end,
	icon = "Interface\\Icons\\INV_Misc_PocketWatch_01",
	OnEnter = function(self)
		tooltip = QTC:Acquire("InstanceInfoTT", 7, "LEFT", "CENTER", "RIGHT", "CENTER", "RIGHT", "CENTER", "RIGHT");
		tooltip:SetAutoHideDelay(2, self);
		drawTooltip();
		tooltip:SmartAnchorTo(self);
		tooltip:EnableMouse();
		tooltip:Show();
	end,
})

------------------------------------------------------------------------------------------------------
-- Event that fires:
--	when we leave the world
--	when the instance we are in gets updated
--	when the instance we are in starts up
--	when the instance we are in is done starting
------------------------------------------------------------------------------------------------------
local function OnInstanceEvent()
	InstanceInfo:SaveInstanceInfo();
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is first loaded
------------------------------------------------------------------------------------------------------
function InstanceInfo:OnInitialize()
	-- Register the database
	InstanceInfo.db = LibStub("AceDB-3.0"):New("InstanceInfoFuDB", InstanceInfo.defaults);

	-- Add info to the Options array for the Profile Options
	InstanceInfo.options.args.profile.args.options = LibStub("AceDBOptions-3.0"):GetOptionsTable(InstanceInfo.db);
	InstanceInfo.options.args.profile.args.options.name = L["Options profile"];
	InstanceInfo.options.args.profile.args.options.desc = L["Saved profile for InstanceInfo options"];
	InstanceInfo.options.args.profile.args.options.order = 2;

	-- Create options table and stuff
	LibStub("AceConfig-3.0"):RegisterOptionsTable("InstanceInfo", InstanceInfo.options, "instanceinfo")
	InstanceInfo.optFrame = AceConfig:AddToBlizOptions("InstanceInfo", "InstanceInfo")
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is enabled
------------------------------------------------------------------------------------------------------
function InstanceInfo:OnEnable()
	-- Register for important events
	InstanceInfo:RegisterEvent("PLAYER_ENTERING_WORLD",	OnInstanceEvent);
	InstanceInfo:RegisterEvent("PLAYER_LEAVING_WORLD",	OnInstanceEvent);
	InstanceInfo:RegisterEvent("UPDATE_INSTANCE_INFO",	OnInstanceEvent);
	InstanceInfo:RegisterEvent("INSTANCE_BOOT_START",	OnInstanceEvent);
	InstanceInfo:RegisterEvent("INSTANCE_BOOT_STOP",	OnInstanceEvent);

	-- Ask for instance info
	RequestRaidInfo();
	
	-- Update text
	self:UpdateBrokerText();
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is disabled
------------------------------------------------------------------------------------------------------
function InstanceInfo:OnDisable()
	-- Remove registration for our events
	InstanceInfo:UnregisterEvent("PLAYER_LEAVING_WORLD")
	InstanceInfo:UnregisterEvent("UPDATE_INSTANCE_INFO")
	InstanceInfo:UnregisterEvent("INSTANCE_BOOT_START")
	InstanceInfo:UnregisterEvent("INSTANCE_BOOT_STOP")
end

------------------------------------------------------------------------------------------------------
-- Update saved text
------------------------------------------------------------------------------------------------------
function InstanceInfo:OnUpdate()	
	-- Update Broker text
	self:UpdateBrokerText();
end

------------------------------------------------------------------------------------------------------
-- Broker text update code
------------------------------------------------------------------------------------------------------
function InstanceInfo:UpdateBrokerText()
	local numInstances = 0;
	
	if (InstanceInfo.db.profile.showSelf) then
		 numInstances = GetNumSavedInstances();
	else
		for player, playerinfo in pairs(InstanceInfo.db.realm.info) do
			if (playerinfo ~= nil) then
				local nMax = #(playerinfo.instances);
				for i = 1, nMax do
					local instance = playerinfo.instances[i];
					if (instance ~= nil) then
						numInstances = numInstances + 1;
					end
				end
			end
		end
	end
	
	if (numInstances == 0) then
		-- We're not saved to anything
		dataobj.text = "|cff00ff00"..L["No Saved Instances"].."|r";
	else
		dataobj.text = "|cff00ff00"..L["Saved Instances"]..": |r"..numInstances;
	end
end

------------------------------------------------------------------------------------------------------
-- Instance information saving
------------------------------------------------------------------------------------------------------
function InstanceInfo:SaveInstanceInfo()
	if (UnitLevel("player") < tonumber(InstanceInfo.db.profile.minLevel)) then
		-- We don't want to store instance info for low-level players
		-- Won't really be useful till WotLK, but meh =p
		return;
	end

	local player = UnitName("player");
	if (InstanceInfo.db.realm.info[player] == nil) then
		InstanceInfo.db.realm.info[player] = {};
	end
	
	-- Store a bunch of fun stuff
	InstanceInfo.db.realm.info[player].player	= player;
	InstanceInfo.db.realm.info[player].engClass	= select(2, UnitClass("player"));
	InstanceInfo.db.realm.info[player].curtime	= time();
	
	-- Store name of instances we are saved to
	InstanceInfo.db.realm.info[player].instances	= {};

	local numInstances = GetNumSavedInstances();
	for i = 1, numInstances do
		local instance = {};
		instance.name, instance.ID, instance.remaining, instance.difficulty = GetSavedInstanceInfo(i);
		InstanceInfo.db.realm.info[player].instances[i] = instance;
	end

	table.sort(InstanceInfo.db.realm.info[player].instances, function(a, b) return a.name < b.name end);
	table.sort(InstanceInfo.db.realm.info, function(a,b) return a.player < b.player; end);
	
	-- Update our text
	self:UpdateBrokerText();

	return true;
end

------------------------------------------------------------------------------------------------------
-- Having a function with msg being passed to it allows us to create /commands later if desired
-- If no msg is passed, open the GUI, else accept a chat command.
------------------------------------------------------------------------------------------------------
function InstanceInfo:ChatCommand(msg)
	if not input or input:trim() == "" then
		-- We're opening config dialogue
		LibStub("AceConfigDialog-3.0"):Open("InstanceInfo");
	else
		-- We have a chat command
		LibStub("AceConfigCmd-3.0").HandleCommand(InstanceInfo, "instanceinfo", msg);
	end
	
end